@@ -1,28 +1,28 @@
-from __future__ import division, print_function
-
+# coding: utf-8
 """
 Low-level objects providing an abstraction for the objects involved in the
 calculation.
 """
+from __future__ import unicode_literals, division, print_function
 
 import collections
 import os
 import abc
+import six
 import numpy as np
 import pymatgen.core.units as units
 
 from pprint import pformat
 from monty.design_patterns import singleton
-from pymatgen.util.string_utils import is_string
-from pymatgen.core.design_patterns import Enum, AttrDict
+from monty.string import is_string
+from monty.collections import AttrDict
+from pymatgen.core.design_patterns import Enum
 from pymatgen.core.units import ArrayWithUnit
 from pymatgen.serializers.json_coders import PMGSONable
 from pymatgen.symmetry.finder import SymmetryFinder
 from pymatgen.core.structure import Structure, Molecule
 from pymatgen.io.smartio import read_structure
-
 from .netcdf import structure_from_etsf_file
-import six
 
 
 def contract(s):
