@@ -1,6 +1,6 @@
-from __future__ import division, print_function
-
+# coding: utf-8
 """Tools to compute equations of states with different models."""
+from __future__ import unicode_literals, division, print_function
 
 import collections
 import numpy as np
