@@ -1,8 +1,14 @@
-from __future__ import unicode_literals
-
+# coding: utf-8
 """
 This module provides utility classes for string operations.
 """
+from __future__ import unicode_literals
+import re
+import sys
+import fnmatch
+
+from six.moves import zip
+from monty.string import list_strings
 
 
 __author__ = "Shyue Ping Ong"
@@ -13,11 +19,6 @@ __email__ = "shyuep@gmail.com"
 __status__ = "Production"
 __date__ = "$Sep 23, 2011M$"
 
-import re
-import sys
-import fnmatch
-from six.moves import zip
-
 
 def generate_latex_table(results, header=None, caption=None, label=None):
     """
@@ -52,7 +53,7 @@ def str_delimited(results, header=None, delimiter="\t"):
     """
     Given a tuple of tuples, generate a delimited string form.
     >>> results = [["a","b","c"],["d","e","f"],[1,2,3]]
-    >>> print str_delimited(results,delimiter=",")
+    >>> print(str_delimited(results,delimiter=","))
     a,b,c
     d,e,f
     1,2,3
@@ -75,7 +76,7 @@ def str_aligned(results, header=None):
     """
     Given a tuple, generate a nicely aligned string form.
     >>> results = [["a","b","cz"],["d","ez","f"],[1,2,3]]
-    >>> print str_aligned(results)
+    >>> print(str_aligned(results))
     a    b   cz
     d   ez    f
     1    2    3
@@ -191,38 +192,6 @@ def pprint_table(table, out=sys.stdout, rstrip=False):
         out.write("\n")
 
 
-def is_string(s):
-    """True if s behaves like a string (duck typing test)."""
-    try:
-        s + " "
-        return True
-
-    except TypeError:
-        return False
-
-
-def list_strings(arg):
-    """
-    Always return a list of strings, given a string or list of strings as
-    input.
-
-    :Examples:
-
-    >>> list_strings('A single string')
-    ['A single string']
-
-    >>> list_strings(['A single string in a list'])
-    ['A single string in a list']
-
-    >>> list_strings(['A','list','of','strings'])
-    ['A', 'list', 'of', 'strings']
-    """
-    if is_string(arg):
-        return [arg]
-    else:
-        return arg
-
-
 def stream_has_colours(stream):
     """
     True if stream supports colours. Python cookbook, #475186
